<?php
/**
 * Class definition file for the Point Of Marketing API
 *
 * The primary controller for POM API calls.
 *
 * @package POMAPI
 * @author Jeremy Miller
 * @copyright 2015 Repeat Returns
 * @version 2.0.1
 * @since 2.0.0
 */
namespace POMAPI;

include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Object.php';
include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Response.php';
include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Chain.php';
include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Member.php';
include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Promotion.php';
include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Store.php';
include __DIR__ . DIRECTORY_SEPARATOR . 'POM_Transaction.php';

class POM {
  const BASE_URL = 'http://api.pointofmarketing.net:3357/';
  private $curl = false;
  private $_username = false;
  private $_password = false;
  private $_clerk = false;
  /**
   * Prepares this instance for communicating with the POM API server.
   *
   * @param $username The username assigned to this account.
   * @param $password The password assigned to this account.
   * @param $clerk The clerk id to use for this instance's calls.
   */
  public function __construct($username, $password, $clerk) {
    $this->_username = $username;
    $this->_password = $password;
    $this->_clerk = $clerk;
  }
  /**
   * Send a GET request.
   *
   * @param $resource The resource to access.
   * @param $resource_id The id of the resource to access.
   * @param $sub_resource The resource for this id to access.
   * @return POM_Response The response from the server
   */
  public function get($resource, $resource_id, $sub_resource = '') {
    try {
      return $this->_prepareAndSend($resource, $resource_id, $sub_resource, '');
    } catch (\Exception $e) {
      $response = new POM_Response(array('code'=>400, 'msg'=>'Unable to process request'));
    }
  }
  /**
   * Send a PUT request.
   *
   * @param $resource The resource to access.
   * @param $resource_id The id of the resource to access.
   * @param $fields The POM_Object to update
   * @return POM_Response The response from the server
   */
  public function put($resource, $resource_id, $fields) {
    try {
      return $this->_prepareAndSend($resource, $resource_id, '', $fields->serialize(), 'PUT');
    } catch (\Exception $e) {
      $response = new POM_Response(array('code'=>400, 'msg'=>'Unable to process request'));
    }
  }
  /**
   * Send a POST request.
   *
   * @param $resource The resource to access.
   * @param $fields The POM_Object to create
   * @return POM_Response The response from the server
   */
  public function post($resource, $fields) {
    try {
      return $this->_prepareAndSend($resource, '', '', $fields->serialize());
    } catch (\Exception $e) {
      $response = new POM_Response(array('code'=>400, 'msg'=>'Unable to process request'));
    }
  }
  /**
   * Send a DELETE request.
   *
   * @param $resource The resource to access.
   * @param $resource_id The id of the resource to access.
   * @return POM_Response The response from the server
   */
  public function delete($resource, $resource_id) {
    try {
      return $this->_prepareAndSend($resource, $resource_id, '', '', 'DELETE');
    } catch (\Exception $e) {
      $response = new POM_Response(array('code'=>400, 'msg'=>'Unable to process request'));
    }
  }
  function _prepareAndSend($resource, $resource_id, $sub_resource, $fields, $method = false) {
    try {
      $bytes = openssl_random_pseudo_bytes(12);
      $nonce = bin2hex($bytes);
      $headers = array();
      if (strlen($fields) == 0) {
        $payload = $resource_id;
      } else if ($method == 'GET' && strlen($sub_resource) == 0) {
        $sub_resource = $fields;
        $payload = substr($fields, 1);
      }
      else {
        $payload = $fields;
      }
      
      $headers[] = 'pom-username: ' . $this->_username;
      $headers[] = 'pom-nonce: ' . $nonce;
      if (strlen($this->_clerk) > 0) {
        $headers[] = 'pom-clerk: ' . $this->_clerk;
      }
      $headers[] = 'pom-hash: ' . $this->calculateHash($nonce, $this->_username, $this->_password, $payload);
      
      $result = $this->_send(POM::BASE_URL . $resource . '/' . $resource_id . $sub_resource, $fields, $headers, $method);
      
      $response = new POM_Response(json_decode($result));

      $response->remote_url = POM::BASE_URL . $resource . '/' . $resource_id . $sub_resource;
      $response->remote_payload = $fields;
      
      return $response;
    } catch (\Exception $e) {
      throw new \Exception("Could not prepare call.");
    }
  }
  /**
   * Send a call to the POM API using cURL.
   *
   * @param $url The resource/resource_id/related_resource portion of the URL.
   * @param $fields Fields to be sent as POST fields
   * @param $headers Additional headers to set for the call
   * @param $method Set for PUT or DELETE calls.  For GET and POST calls, this value is automatically determined.
   */
  private function _send($url, $fields = '', $headers = array(), $method = false) {
    try {
      if ($this->curl === false) {
        $this->curl = curl_init();
      }
      if ($this->curl === false) {
        throw new \Exception("Connection to CURL failed.");
      }
    } catch (\Exception $e) {
      throw new \Exception("Connection failed.");
    }
    if (count($headers) > 0) {
      curl_setopt($this->curl, CURLOPT_HTTPHEADER, $headers);
    }
    if (strlen($fields) > 0 && $method !== 'GET') {
      curl_setopt($this->curl, CURLOPT_POST, 1);
      curl_setopt($this->curl, CURLOPT_POSTFIELDS, $fields);
    }
    if ($method !== false) {
      curl_setopt($this->curl, CURLOPT_CUSTOMREQUEST, $method); //PUT or DELETE
    }
    curl_setopt($this->curl, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($this->curl, CURLOPT_URL, $url);
    curl_setopt($this->curl, CURLOPT_RETURNTRANSFER, 1);
    return curl_exec($this->curl);
  }
  function calculateHash($nonce, $username, $password, $payload) {
    return hash('sha256', $nonce . ':' . $username . ':' . $password . ':' . $payload);
  }
}
?>